@@ -1,6 +1,7 @@
 import binascii
 from concurrent.futures import ThreadPoolExecutor
 from pathlib import Path
+import logging
 
 from .assets import KNOWN_ASSETS, AssetStore, EXTRACTED_DIR, OVERRIDES_DIR
 
@@ -19,14 +20,12 @@ def main():
     import argparse
 
     parser = argparse.ArgumentParser(description="Extract Spelunky 2 Assets.")
-    parser.add_argument(
-        "--lazy", default=False, action="store_true",
-        help="Whether to be lazy when rebuilding and only extract files that don't exist."
-    )
     parser.add_argument("exe", type=argparse.FileType("rb"), help="Path to Spel2.exe")
 
     args = parser.parse_args()
 
+    logging.basicConfig(format="%(levelname)s - %(message)s", level=logging.INFO)
+
     asset_store = AssetStore.load_from_file(args.exe)
     seen = {}
 
@@ -37,34 +36,36 @@ def main():
         (OVERRIDES_DIR / dir_).mkdir(parents=True, exist_ok=True)
         (OVERRIDES_DIR / ".compressed" / dir_).mkdir(parents=True, exist_ok=True)
 
-    def extract_single(filename):
+    for filename in KNOWN_ASSETS:
         asset = asset_store.find_asset(filename)
         name_hash = asset_store.filename_hash(filename)
         if asset is None:
-            print("Asset {} not found with hash {!r}...".format(
+            logging.warning("Asset %s not found with hash %s...",
                 filename.decode(),
                 binascii.hexlify(name_hash)
-            ))
-            return
+            )
+            continue
 
         asset.filename = filename
         seen[asset.name_hash] = asset
 
         filepath = Path(filename.decode())
-        if args.lazy and (EXTRACTED_DIR / filepath).exists():
-            print(f"{filepath} already found. Skipping...")
-            return
+        asset.load_data(args.exe)
 
-        print("Extracting {}... ".format(filepath))
-        asset.extract(EXTRACTED_DIR, args.exe, asset_store.key)
+    def extract_single(asset):
+        try:
+            logging.info("Extracting %s... ", asset.filename.decode())
+            asset.extract(EXTRACTED_DIR, asset_store.key)
+        except Exception as err:
+            logging.error(err)
 
     pool = ThreadPoolExecutor()
-    pool.map(extract_single, KNOWN_ASSETS)
+    pool.map(extract_single, seen.values())
 
     for asset in sorted(asset_store.assets, key=lambda a: a.offset):
         name_hash = asset_store.filename_hash(asset.filename)
         if asset.name_hash not in seen:
-            print("Un-extracted Asset", asset)
+            logging.warning("Un-extracted Asset %s", asset)
 
 
 if __name__ == '__main__':
